/*----------------------------------------------------------------------------
 *
 * file   : capture.c
 *
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 *
 *---------------------------------------------------------------------------*/

#include <string.h>
#include <X11/cursorfont.h>
#include "globals.h"

Window getWindow();
void   doCapture();
void   doDelay();

static int increment = 0;

static void captureArea(),
	    captureWindow(),
	    captureFullScreen();

/* if delay seconds are specified before capturing, then 
   display count down, and delay the beginning of capture */

void
doDelay()
{
    int i;
    char sec[32];
    XDefineCursor(display,XtWindow(toplevel),busyCursor);
    for (i=delaySeconds;i>=0;i--) {
	sprintf(sec,"%2d sec before snap ",i);
	setMenuTitle(sec);
	sleep(1);
    }
    XDefineCursor(display,XtWindow(toplevel),windowCursor);
    resetMenuTitle();
}

void 
doCaptureCB()
{
    
    switch(captureMode) {
    case CaptureArea :
    case CaptureNTSC :
    case CapturePAL  :
	captureArea();
	break;
    case CaptureWindow :
	captureWindow();
	break;
    case CaptureFullScreen :
	captureFullScreen();
	break;
    }
}

void
doCapture(x,y,width,height)
int x,y,width,height;
{
    char str[526];

    sprintf(str,"/usr/sbin/scrsave %s %d %d %d %d",
				fileToSave,
				x, 
				x+width, 
				(screenHeight-y),
				(screenHeight-(y+height)));


    strcpy(fileToPreview,fileToSave);
    system(str);
    XBell(display,25);

    /* if auto increment is on, update file name */
    if (autoIncrement) {
	char *c;
	char extension[10];
	increment++;

	sprintf(fileToSave,"%s%03d%s",baseFileName,increment,fileExtension);
        XtVaSetValues(fileSaveTextW,XmNvalue,fileToSave,NULL);

    }

    previewReady = 1;
    updateMenus();
}

static void
captureWindow()
{
    Window win;
    Window ignore_win;
    int x,y;
    unsigned int width,height,border,depth;

    XDefineCursor(display,XtWindow(toplevel),windowCursor);
    win = getWindow();
    if (win == rootWindow) {
	XBell(display,25);
	return;
    }
    XRaiseWindow(display,win);

    XGetGeometry(display,win,&ignore_win,&x,&y, &width,&height, &border,&depth);
    if (x < 0) x = 0;
    if (y < 0) y = 0;

    if (delaySeconds) {
	doDelay();
    }

    /* make the ui disappear while snaphsot in progress */
    XtUnmanageChild(settingDialog);
    XUnmapWindow(display,XtWindow(toplevel));
    XSync(display,True);
    doCapture(x,y+2,width-2,height-2);
    XMapWindow(display,XtWindow(toplevel));

    XDefineCursor(display,XtWindow(toplevel),cameraCursor);
}

static void
captureFullScreen()
{
    if (delaySeconds) {
	doDelay();
    }
    /* make the ui disappear while snaphsot in progress */
    XtUnmanageChild(settingDialog);
    XUnmapWindow(display,XtWindow(toplevel));
    XSync(display,True);
    doCapture(0,0,screenWidth,screenHeight);
    XMapWindow(display,XtWindow(toplevel));
}

static void
captureArea()
{
    if (delaySeconds) {
	XDefineCursor(display,fullScreen.window,busyCursor); 
	XDefineCursor(display,XtWindow(toplevel),busyCursor);
	doDelay();
	XDefineCursor(display,XtWindow(toplevel),cameraCursor);
	XDefineCursor(display,fullScreen.window,cameraCursor); 
    }
    setMenuTitle("Snap in progress....");
    doCapture(sweepArea.x+1,sweepArea.y+2,sweepArea.width-2,sweepArea.height-2);
    resetMenuTitle();
}

void
previewCB()
{
    char str[526];
    sprintf(str,"ipaste %s &",fileToPreview);
    system(str);
}

/* window selection routine */
Window
getWindow()
{

  int status;
  Window target_win = None;
  int buttons = 0;
  XEvent event;

  /* Grab the pointer using target cursor */
  status = XGrabPointer(display, rootWindow, False,
                        ButtonPressMask|ButtonReleaseMask, GrabModeSync,
                        GrabModeAsync, rootWindow, windowCursor, CurrentTime);

  /* Let the user select a window... */
  while ((target_win == None) || (buttons != 0)) {
      /* allow one more event */
      XAllowEvents(display, SyncPointer, CurrentTime);
      XWindowEvent(display, rootWindow, ButtonPressMask|ButtonReleaseMask, &event);
      switch (event.type) {
      case ButtonPress:
          if (target_win == None)
          {
            target_win = event.xbutton.subwindow; /* window selected */
            if (target_win == None) target_win = rootWindow;
          }
          buttons++;
          break;
      case ButtonRelease:
          if (buttons > 0) /* there may have been some down before we started */
              buttons--;
          break;
      }
  }

  XUngrabPointer(display, CurrentTime);      /* Done with grab mode */

  return(target_win);

}
