/*----------------------------------------------------------------------------
 *
 * file   : event.c
 *
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 *
 *---------------------------------------------------------------------------*/

#include <X11/keysym.h>
#include <X11/StringDefs.h>

#include "globals.h"

extern int findTrackMode();
void fineTuneSweep(),
     readjustPointer();

static int keyAccel = 0;
static int fineTuneDelta = 1;

/* The event loop */

void
doEvents()
{
    XEvent event;
    char string[31];
    XComposeStatus  composeStatus;
    KeySym          keysym;
    int             bpressX,bpressY;
    int             button1Down = 0;
    int             prevTrackMode = -1;

    while (1) {
        XtAppNextEvent(appContext, &event);

	if (event.xany.window == fullScreen.window) {
 	    switch(event.type) {
	    case Expose :
		drawSweep();
		break;
	    case ButtonPress :
		switch(event.xbutton.button) {
		case Button1 :
		    button1Down = 1;
		    updateAnchorPoint(event.xbutton.x,event.xbutton.y,&bpressX,&bpressY);
		    oldX = bpressX;
		    oldY = bpressY;
		    break;
		case Button2 :
		    sweeping = 0;
		    disableSweep();
		    break;
		case Button3 :
		    break;
		}
		prevTrackMode = -1;
		break;
	    case ButtonRelease :
		switch(event.xbutton.button) {
		case Button1 :
		    updateMenus();
	            aspectRatio = (float)sweepArea.width/(float)sweepArea.height;
		    button1Down = 0;
		    break;
		case Button2 :
		    disableSweep();
		    break;
		case Button3 :
		    break;
		}
		break;
	    case KeyPress :
		XLookupString((XKeyEvent *)&event,string,30,&keysym,&composeStatus);
		switch(keysym) {
		case XK_Up :
		case XK_Right :
		case XK_Left :
		case XK_Down :
		    fineTuneSweep(keysym,trackMode);
		    drawSweep();
		    break;
		case XK_Shift_L :
		case XK_Shift_R :
		    keyAccel++;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		case XK_Control_L :
		case XK_Control_R :
		    keyAccel++;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		}
		break;
	    case KeyRelease :
		XLookupString((XKeyEvent *)&event,string,30,&keysym,&composeStatus);
		switch(keysym) {
		case XK_Shift_L :
		case XK_Shift_R :
		    keyAccel--;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		case XK_Control_L :
		case XK_Control_R :
		    keyAccel--;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		}
		break;
	    case MotionNotify :
		if (!button1Down) {
		    trackMode = findTrackMode(event.xmotion.x,event.xmotion.y);
		    if (prevTrackMode != trackMode) {
		        prevTrackMode = trackMode;
                        XDefineCursor(display,fullScreen.window,trackCursor[trackMode]);
		    }
		}
		else {
		    updateSweep(bpressX,bpressY,event.xmotion.x,event.xmotion.y);
                    drawSweep();
                }
		break;
	    }
	}
/*
	else if (event.xany.window == XtWindow(toplevel)) {
*/
	else {
 	    switch(event.type) {
	    case KeyPress :
		XLookupString((XKeyEvent *)&event,string,30,&keysym,&composeStatus);
		switch(keysym) {
		case XK_Up :
		case XK_Right :
		case XK_Left :
		case XK_Down :
		    if (sweeping) {
		        fineTuneSweep(keysym,trackMode);
		        drawSweep();
		    }
		    break;
		case XK_Shift_L :
		case XK_Shift_R :
		    keyAccel++;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		case XK_Control_L :
		case XK_Control_R :
		    keyAccel++;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		case XK_F1 :
		    if (XtIsSensitive(doCaptureMenuItemW))
		        doCaptureCB();
		    break;
		case XK_F2 :
		    if (XtIsSensitive(sweepAreaMenuItemW))
		        enableSweep();
		    break;
		case XK_Escape :
		    exit(1);
		    break;
		}
		break;
	    case KeyRelease :
		XLookupString((XKeyEvent *)&event,string,30,&keysym,&composeStatus);
		switch(keysym) {
		case XK_Shift_L :
		case XK_Shift_R :
		    keyAccel--;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		case XK_Control_L :
		case XK_Control_R :
		    keyAccel--;
		    fineTuneDelta = 1 + 2*keyAccel;
		    break;
		}
		break;
	    case ButtonRelease :
		if (sweeping) {
		    drawSweep();
		    XFlush(display);
		}
		break;
	    case Expose :
		if (!sweeping)
		    drawOnePixelWindows();
		break;
	    }
	}
	XtDispatchEvent(&event);
    } /* while */
}

void
readjustPointer()
{
    switch(trackMode) {
    case TrackBottomLeft :
	XWarpPointer(display,fullScreen.window,fullScreen.window,
		     0,0,screenWidth,screenHeight,
		     sweepArea.x,sweepArea.y+sweepArea.height);
	break;
    case TrackTopLeft :
	XWarpPointer(display,fullScreen.window,fullScreen.window,
		     0,0,screenWidth,screenHeight,
		     sweepArea.x,sweepArea.y);
	break;
    case TrackTopRight :
	XWarpPointer(display,fullScreen.window,fullScreen.window,
		     0,0,screenWidth,screenHeight,
		     sweepArea.x+sweepArea.width,sweepArea.y);
	break;
    case TrackBottomRight :
	XWarpPointer(display,fullScreen.window,fullScreen.window,
		     0,0,screenWidth,screenHeight,
		     sweepArea.x+sweepArea.width,sweepArea.y+sweepArea.height);
	break;
    }
}

/* fine tuning of sweep area via keyboard arrow keys */

void
fineTuneSweep(keysym,tmode)
KeySym          keysym;
int             tmode;
{
    switch(keysym) {
    case XK_Left :
	switch(tmode) {
	case TrackCenter :
	    sweepArea.x -= fineTuneDelta;
	    break;
        case TrackLeft :
        case TrackTopLeft :
        case TrackBottomLeft :
	    sweepArea.x -= fineTuneDelta;
	    sweepArea.width += fineTuneDelta;
	    break;
        case TrackRight :
        case TrackTopRight :
        case TrackBottomRight :
	    sweepArea.width -= fineTuneDelta;
	    break;
	}
	XWarpPointer(display,fullScreen.window,None,
		     0,0,screenWidth,screenHeight,-fineTuneDelta,0);
	if (keepAspectRatio || squareConstraint) {
	    sweepArea.height = sweepArea.width/aspectRatio;
	    readjustPointer();
	}
	break;
    case XK_Right :
	switch(tmode) {
	case TrackCenter :
	    sweepArea.x += fineTuneDelta;
	    break;
        case TrackRight :
        case TrackTopRight :
        case TrackBottomRight :
	    sweepArea.width += fineTuneDelta;
	    break;
        case TrackLeft :
        case TrackTopLeft :
        case TrackBottomLeft :
	    sweepArea.x += fineTuneDelta;
	    sweepArea.width -= fineTuneDelta;
	    break;
	}
	XWarpPointer(display,fullScreen.window,None,
		     0,0,screenWidth,screenHeight,fineTuneDelta,0);
	if (keepAspectRatio || squareConstraint) {
	    sweepArea.height = sweepArea.width/aspectRatio;
	    readjustPointer();
	}
	break;
    case XK_Up :
	switch(tmode) {
	case TrackCenter :
	    sweepArea.y -= fineTuneDelta;
	    break;
        case TrackTop :
        case TrackTopLeft :
        case TrackTopRight :
	    sweepArea.y -= fineTuneDelta;
	    sweepArea.height += fineTuneDelta;
	    break;
        case TrackBottom :
        case TrackBottomRight :
        case TrackBottomLeft :
	    sweepArea.height -= fineTuneDelta;
	    break;
	}
	XWarpPointer(display,fullScreen.window,None,
		     0,0,screenWidth,screenHeight,0,-fineTuneDelta);
	if (keepAspectRatio || squareConstraint) {
	    sweepArea.width = sweepArea.height*aspectRatio;
	    readjustPointer();
        }
	break;
    case XK_Down :
	switch(tmode) {
	case TrackCenter :
	    sweepArea.y += fineTuneDelta;
	    break;
        case TrackTop :
        case TrackTopLeft :
        case TrackTopRight :
	    sweepArea.y += fineTuneDelta;
	    sweepArea.height -= fineTuneDelta;
	    break;
        case TrackBottom :
        case TrackBottomRight :
        case TrackBottomLeft :
	    sweepArea.height += fineTuneDelta;
	    break;
	}
	XWarpPointer(display,fullScreen.window,None,
		     0,0,screenWidth,screenHeight,0,fineTuneDelta);
	if (keepAspectRatio || squareConstraint) {
	    sweepArea.width = sweepArea.height*aspectRatio;
	    readjustPointer();
        }
	break;
    }
}
