/*----------------------------------------------------------------------------
 *
 * file   : globals.h for screenshot
 *
 * Author : Yusuf Attarwala
 * Date   : Sep 96 
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>

#ifdef DEFINE_GLOBALS
#define GLOBAL
#define INIT(x) = x
#else
#define GLOBAL extern
#define INIT(x)
#endif

typedef struct _menuItem {
    Widget             widget;
    char              *label;
    WidgetClass       *class;
    char               mnemonic;
    char              *accelerator;
    char              *accelText;
    void             (*callback)();
    XtPointer          callbackData;
    struct _menuItem  *subItems;    /* rollover menus if not NULL */
} MenuItem;

typedef struct _box2d {
    int x,y;
    Dimension width,height;
} Box2D;

typedef struct _window {
    Window 	window;
    GC          drawgc,
		erasegc,
		draw2gc;
    Box2D       bounds;
} WindowType;

#define ABS(a)   (((a) >= 0) ? (a) : -(a))
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b))

GLOBAL Display        *display;
GLOBAL long            screen;
GLOBAL XtAppContext    appContext;
GLOBAL XtTranslations  transTable;
GLOBAL Widget          topmostlevel,toplevel;
GLOBAL Window          rootWindow;
GLOBAL Dimension       screenHeight, screenWidth;

GLOBAL Widget          settingDialog;
GLOBAL Widget          helpDialog;
GLOBAL Widget          incrementToggleW,
		       aspectToggleW,
		       squareToggleW,
		       fileSaveTextW,delaySecondsW;
GLOBAL Widget          menuTitleW,
		       sweepAreaMenuItemW,
		       doCaptureMenuItemW,
		       previewMenuItemW;
                       
GLOBAL Box2D           sweepArea;

GLOBAL Cursor          crossHair,
		       busyCursor,
                       cameraCursor,
		       windowCursor;

#define                TrackOutside      0
#define                TrackCenter       1
#define                TrackTopLeft      2
#define                TrackTop          3
#define                TrackTopRight     4 
#define                TrackRight        5
#define                TrackBottomRight  6
#define                TrackBottom       7
#define                TrackBottomLeft   8
#define                TrackLeft         9

GLOBAL Cursor          trackCursor[10];           /* cursors for tracking */
GLOBAL int             trackMode;

GLOBAL XColor          redColor,                  /* some useful colors   */
		       greenColor,
		       blackColor,
		       whiteColor,
		       greyColor,
		       brownColor;

GLOBAL XFontStruct     *mediumNormalFont;

GLOBAL Pixmap          cameraPixmap;
#define LeftWindow     0
#define RightWindow    1
#define BottomWindow   2
#define TopWindow      3

GLOBAL WindowType      fullScreen,     /* full screen overlay */
                       redline[4];     /* 4 one pixel overlay windows */

GLOBAL int             autoIncrement;  /* increment file count after snap */
GLOBAL int             keepAspectRatio;
GLOBAL int             squareConstraint;
GLOBAL float           aspectRatio;    /* x over y */
GLOBAL int             delaySeconds;   /* wait so many seconds before snap*/
GLOBAL int             previewReady;
GLOBAL int             sweeping;

#define CaptureArea        0 
#define CaptureWindow      1
#define CaptureFullScreen  2
#define CaptureNTSC        3
#define CapturePAL         4

GLOBAL  int  	       captureMode;
GLOBAL  int            oldX,oldY;

GLOBAL char            baseFileName[256];
GLOBAL char            fileExtension[10];
GLOBAL char            fileToSave[256];
GLOBAL char            fileToPreview[256];

GLOBAL void            voidFunction();
GLOBAL Widget          createLabel();

/* these should be on the last lines */
#undef INIT
#undef GLOBAL
