/*----------------------------------------------------------------------------
 *
 * file   : main.c for screenshot
 *
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>

#define DEFINE_GLOBALS 1
#include "globals.h"
#undef DEFINE_GLOBALS
#include <Xm/DialogS.h>

/* currently commandline args not used, 
   but this is a place holder for future enhancements */

typedef struct _cmdLine {
    String     status;           
} CmdLine, *CmdLinePtr;

CmdLine commandLine;

static XtResource resources[] = {
    {"status",    "Status",   XtRString,  sizeof(String),
      XtOffset(CmdLinePtr,status), XtRString, "0",},
};

static XrmOptionDescRec options[] = {
    {"-s",      "status",  XrmoptionSepArg, NULL},
};

static void createCameraCursor();
static void createWindowCursor();
static void createCustomCursors();

static String fallbackResources[] = {
    "*useSchemes:                  all",
    "*sgiMode:                     true",
    "*background:                  grey",
    "*XmLabel.traversalOn:         False",
    "*fillOnSelect:                True",
    "*selectColor:                 yellow",
    "*menub*fontList:              -adobe-helvetica-*-o-normal-*-14-*-*-*-*-*-*-*",
    "*label*fontList:              -adobe-helvetica-*-o-normal-*-12-*-*-*-*-*-*-*",
    NULL
};

/* function declaration */
void main();
static void initX(),
	    initGlobals();

void
main(argc,argv)
int argc;
char **argv;
{
    initX(argc,argv);
    initGlobals();
    createUI();
    doEvents();    
}

static void
initX(argc,argv)
int argc;
char **argv;
{
    int n;
    Arg alist[25];
    XFontStruct *genericFont;

    n = 0;
    XtSetArg(alist[n],XmNmappedWhenManaged,False); n++;
    topmostlevel = XtAppInitialize(&appContext,"ScreenShot",
                               options,XtNumber(options),
                               &argc,argv,
                               fallbackResources,
                               alist, n);

    /* make a transient shell to avoid window manager intervention later on */

    n = 0;
    XtSetArg(alist[n],XmNmwmDecorations,0);n++;   /* borderless window */
    XtSetArg(alist[n],XmNallowResize,False);n++;
    XtSetArg(alist[n],XmNmaxWidth,140);n++;
    XtSetArg(alist[n],XmNmaxHeight,40);n++;
    XtSetArg(alist[n],XmNminWidth,132);n++;
    XtSetArg(alist[n],XmNminHeight,25);n++;
    toplevel = XtCreatePopupShell("toplevel",transientShellWidgetClass,
				   topmostlevel,alist,n);

    /* cache the usual stuff into globals */
    display    = XtDisplay(toplevel);
    screen     = XDefaultScreen(display);
    rootWindow = DefaultRootWindow(display);
    XtGetApplicationResources(toplevel,&commandLine,
                              resources,XtNumber(resources),
                              NULL,0);

    screenHeight  = DisplayHeight(display,screen);
    screenWidth   = DisplayWidth (display,screen);

    /* save few useful colors */
    {
	XColor temp;
	Colormap cmap = XDefaultColormap(display,screen);
        XAllocNamedColor(display,cmap,"red",  &temp,&redColor);
        XAllocNamedColor(display,cmap,"aquamarine",&temp,&greenColor);
        XAllocNamedColor(display,cmap,"black",&temp,&blackColor);
        XAllocNamedColor(display,cmap,"white",&temp,&whiteColor);
        XAllocNamedColor(display,cmap,"grey", &temp,&greyColor);
        XAllocNamedColor(display,cmap,"red4", &temp,&brownColor);
    }


    /* keep some fonts handy */
    genericFont = XLoadQueryFont(display,"9x15");
    if (!(mediumNormalFont = XLoadQueryFont(display,
                   "-adobe-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*")))
    mediumNormalFont = genericFont;

    /* create and save cursors */
    createCursors();

}

static void
initGlobals()
{
    autoIncrement    = 0;
    keepAspectRatio  = 0;
    squareConstraint = 0;
    delaySeconds     = 0;
    previewReady     = 0;
    captureMode      = CaptureArea;  /* changing this will not affect ui */
    sweeping         = 0;

    /* default file name */
    strcpy(baseFileName,"snap");
    strcpy(fileExtension,".rgb");
    strcpy(fileToSave,"snap.rgb");

    /* area to be captured */
    sweepArea.x        = 0;
    sweepArea.y        = 0;
    sweepArea.width    = 1;
    sweepArea.height   = 1;

    aspectRatio  = sweepArea.width/sweepArea.height;  /* x over y */
}
