/*----------------------------------------------------------------------------
 *
 * overlay.c   
 *  
 * Date   : Sep 96 
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/cursorfont.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <GL/gl.h>                  /* gl includes */
#include <GL/glx.h>                 /* utility library includes */

#include "globals.h"

int           redIndex,
              yellowIndex;

void
createOverlayWindows()
{
    XVisualInfo *vi;
    Colormap ocmap;
    XSetWindowAttributes swa;
    XWindowAttributes winattrs;
    XColor  spix, pixel; 
    int i;

    static int popup[]   = {GLX_LEVEL, 2,  None };
    static int overlay[] = {GLX_LEVEL, 1,  None };

    /* first get the overlay visual */
    if (!(vi = glXChooseVisual (display,screen,overlay))) {
	/* overlay failed, try popup */
	if (!(vi = glXChooseVisual (display,screen,popup))) {
	        printf("could not find overlay or popup visual \n");
		exit(0);
	}
    }

    /* create a colormap */
    ocmap = XCreateColormap(display,rootWindow,vi->visual,AllocNone);

    /* create couple of useful colors */
    {
	XAllocNamedColor(display, ocmap, "red", &spix, &pixel);
	    redIndex = pixel.pixel;
	XAllocNamedColor(display, ocmap, "white", &spix, &pixel);
	    yellowIndex = pixel.pixel;
    }

    swa.colormap              = ocmap;
    swa.border_pixel          = 0;
    swa.background_pixel      = BlackPixel(display, vi->screen);
    swa.win_gravity           = UnmapGravity;
    swa.bit_gravity           = NorthWestGravity;
    swa.event_mask            = StructureNotifyMask;
    swa.override_redirect     = True;
    swa.do_not_propagate_mask = NULL;

    /* create a fullscreen transparent overlay window */

    fullScreen.window  = XCreateWindow (display, rootWindow,
		                        0, 0, screenWidth, screenHeight,
		                        0, vi->depth, InputOutput, vi->visual,
		                        CWBorderPixel|CWOverrideRedirect|
					CWColormap|CWDontPropagate|CWEventMask, &swa);

    /* install the colormap windows property */
    XSetWMColormapWindows(display,fullScreen.window,&fullScreen.window,1);
    /* set event mask */
    XSelectInput(display, fullScreen.window, 
		 ExposureMask|ButtonPressMask|ButtonReleaseMask|PointerMotionMask|KeyPressMask );

    fullScreen.drawgc  = XCreateGC(display,fullScreen.window,0L,(XGCValues *)NULL);
    fullScreen.draw2gc  = XCreateGC(display,fullScreen.window,0L,(XGCValues *)NULL);
    fullScreen.erasegc = XCreateGC(display,fullScreen.window,0L,(XGCValues *)NULL);
    XSetForeground(display,fullScreen.drawgc,redIndex);
    XSetForeground(display,fullScreen.draw2gc,yellowIndex);
    XSetForeground(display,fullScreen.erasegc,0);
    XSetFont(display,fullScreen.draw2gc,mediumNormalFont->fid);
    XSetFont(display,fullScreen.erasegc,mediumNormalFont->fid);

    /* create four more overlay windows */

    for (i=LeftWindow; i <= TopWindow; i++) {
	redline[i].window  = XCreateWindow (display, rootWindow, 0,0,1,1,0,
				     vi->depth, InputOutput, vi->visual,
				     CWBorderPixel|CWOverrideRedirect|
				     CWColormap|CWDontPropagate|CWEventMask, &swa);
	XSetWMColormapWindows(display,redline[i].window,&redline[i].window,1);
        XSelectInput(display, redline[i].window, ExposureMask|ButtonPressMask );

        redline[i].drawgc = XCreateGC(display,redline[i].window,0L,(XGCValues *)NULL);
	XSetForeground(display,redline[i].drawgc,redIndex);
	XDefineCursor(display,redline[i].window,cameraCursor);
	/*
        redline[i].erasegc = XCreateGC(display,redline[i].window,0L,(XGCValues *)NULL);
	XSetForeground(display,redline[i].erasegc,0);
	*/
    }

    XInstallColormap(display,ocmap);
}

