/*----------------------------------------------------------------------------
 *
 * file   : rband.c
 *
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 *
 *---------------------------------------------------------------------------*/

#include "globals.h"

static int redlinesMapped = 0;

#define TickLength 10

void updateSweep(),
     drawOnePixelWindows();
void enableSweep(),
     drawSweep(),
     eraseSweep();

static int topx,topy;
static unsigned int topwidth,topheight;
static int settingManaged;

/* hide the UI window */
int
hideTopLevel()
{
    Window ignore_win;
    unsigned int border,depth;

    XTranslateCoordinates(display,XtWindow(toplevel),rootWindow,0,0,&topx,&topy,&ignore_win);
    if (XtIsManaged(settingDialog)) {
	settingManaged = 1;
        XtUnmanageChild(settingDialog);
    }
    else {
	settingManaged = 0;
    }
    XUnmapWindow(display,XtWindow(toplevel));
}

/* show the UI window */
int 
showTopLevel()
{
    XMapRaised(display,XtWindow(toplevel));
    XMoveWindow(display,XtWindow(toplevel),topx,topy);
    if (settingManaged)
        XtManageChild(settingDialog);
}


void
doSweepCB()
{
    sweeping = 1;
    enableSweep();
}

void
enableSweep()
{
    int i;
    if (redlinesMapped) {
	for (i=LeftWindow;i<=TopWindow;i++)
	    XUnmapWindow(display,redline[i].window);
	redlinesMapped = 0;
    }

    XMapRaised(display,fullScreen.window);

    /* this helps to stack the UI on top of fullscreen overlay window
       so that one can access menus all the time */

    hideTopLevel();
    showTopLevel();

    XDefineCursor(display,fullScreen.window,cameraCursor);
    XDefineCursor(display,XtWindow(toplevel),cameraCursor);

    if (sweepArea.width > 1 || sweepArea.height > 1) {
        drawSweep();
    }
}

void
disableSweep()
{
    int i;
    XUnmapWindow(display,fullScreen.window);
    XDefineCursor(display,XtWindow(toplevel),cameraCursor);

    if (!redlinesMapped) {
	for (i=LeftWindow;i<=TopWindow;i++)
	    XMapRaised(display,redline[i].window);
	redlinesMapped = 1;
    }

    /* left redline */
    XMoveResizeWindow(display,redline[LeftWindow].window,   
		      sweepArea.x,sweepArea.y,1,sweepArea.height);
    /* right redline */
    XMoveResizeWindow(display,redline[RightWindow].window,   
		      sweepArea.x+sweepArea.width,sweepArea.y,1,sweepArea.height);
    /* top redline */
    XMoveResizeWindow(display,redline[TopWindow].window,   
		      sweepArea.x,sweepArea.y,sweepArea.width,1);
    /* bottom redline */
    XMoveResizeWindow(display,redline[BottomWindow].window,   
		      sweepArea.x,sweepArea.y+sweepArea.height,sweepArea.width,1);

    drawOnePixelWindows();
}

void
drawOnePixelWindows()
{
    XDrawLine(display,redline[LeftWindow].window,
		      redline[LeftWindow].drawgc,0,0,0,sweepArea.height);
    XDrawLine(display,redline[RightWindow].window,
		      redline[RightWindow].drawgc,0,0,0,sweepArea.height);
    XDrawLine(display,redline[TopWindow].window,
		      redline[TopWindow].drawgc,0,0,sweepArea.width,0);
    XDrawLine(display,redline[BottomWindow].window,
		      redline[BottomWindow].drawgc,0,0,sweepArea.width,0);
}

void
updateAnchorPoint(bx,by,ax,ay)
int bx,by;
int *ax,*ay;
{
    /* bx,by are where the button was pressed  */

    /* for various track modes, modify the anchor point,
       different than that of original bx,by */

    switch(trackMode) {
    case TrackTopLeft :
    case TrackTop :
    case TrackLeft:
	*ax = sweepArea.x+sweepArea.width;   /* anchor the lower right */
	*ay = sweepArea.y+sweepArea.height;
	break;
    case TrackBottomRight :
    case TrackBottom :
    case TrackRight:
	*ax = sweepArea.x;                   /* anchor the upper left */
	*ay = sweepArea.y;
	break;
    case TrackTopRight :
	*ax = sweepArea.x;                   /* anchor the lower left */
	*ay = sweepArea.y+sweepArea.height;
	break;
    case TrackBottomLeft :
	*ax = sweepArea.x+sweepArea.width;   /* anchor the upper right */
	*ay = sweepArea.y;
	break;
    default :                               /* the original anchor bx,by is good */
	*ax = bx;
	*ay = by;
	break;
    }
}

void
updateSweep(bx,by,mx,my)
int bx,by,mx,my;
{

    /* mx,my are where the button is in motion */
    /* bx,by anchor points */

    static int deltaX, deltaY;
    int left,right,bottom,top;
    int rx,ry;

    rx = ABS(mx-bx);
    ry = ABS(my-by);

    deltaX = mx-oldX;
    deltaY = my-oldY;

    oldX = mx;
    oldY = my;

    /* apply motion constraints */
    switch(trackMode) {
    case TrackLeft:
        my = sweepArea.y;
	if (keepAspectRatio || squareConstraint)
	    my = (my >= by) ? (by + (rx/aspectRatio)) : (by - (rx/aspectRatio));
        break;
    case TrackRight:
        my = sweepArea.y + sweepArea.height;
	if (keepAspectRatio || squareConstraint)
	    my = (my >= by) ? (by + (rx/aspectRatio)) : (by - (rx/aspectRatio));
        break;
    case TrackBottom :
        mx = sweepArea.x + sweepArea.width;
	if (keepAspectRatio || squareConstraint)
	    mx = (mx >= bx) ? (bx + (ry*aspectRatio)) : (bx - (ry*aspectRatio));
        break;
    case TrackTop :
        mx = sweepArea.x;
	if (keepAspectRatio || squareConstraint)
	    mx = (mx >= bx) ? (bx + (ry*aspectRatio)) : (bx - (ry*aspectRatio));
        break;
    case TrackCenter :
        sweepArea.x += deltaX;
	sweepArea.y += deltaY;
        return;
    case TrackTopLeft :
    case TrackTopRight :
	if (keepAspectRatio || squareConstraint) 
	    mx = (mx >= bx) ? (bx + (ry*aspectRatio)) : (bx - (ry*aspectRatio));
	break;
    case TrackBottomLeft :
    case TrackBottomRight :
	if (keepAspectRatio || squareConstraint) 
	    my = (my >= by) ? (by + (rx/aspectRatio)) : (by - (rx/aspectRatio));
	break;
    }

    left   = MIN(bx,mx);
    right  = MAX(bx,mx);
    top    = MIN(by,my);
    bottom = MAX(by,my);

    sweepArea.x      = left;
    sweepArea.y      = top;
    sweepArea.width  = (right-left);
    sweepArea.height = (bottom-top);

    if (squareConstraint && trackMode == TrackOutside) {
	if (sweepArea.width >= sweepArea.height) {
	    sweepArea.width = sweepArea.height;
	}
	else {
	    sweepArea.height = sweepArea.width;
	}
    }
}

void
eraseSweep(x,y,width,height)
int x,y,width,height;
{
    XDrawRectangle(display,fullScreen.window,fullScreen.erasegc,
	           x,y,width,height);
    {
	int cenx,ceny;
	cenx = x + width/2;
	ceny = y + height/2;
        /* erase center lines */
        XDrawLine(display,fullScreen.window,fullScreen.erasegc,
		  x-TickLength,ceny,x,ceny);
        XDrawLine(display,fullScreen.window,fullScreen.erasegc,
		  x+width,ceny, x+width+TickLength,ceny);
        XDrawLine(display,fullScreen.window,fullScreen.erasegc,
		  cenx,y+height,cenx,
		  y+height+TickLength);
    }
    XFillRectangle(display,fullScreen.window,fullScreen.erasegc,
			   x,y-25,390,25);
}

void
drawSweep()
{
    char geometry[200];
    int x,y,width,height;
    static int prevX,prevY,prevWidth,prevHeight;

    eraseSweep(prevX,prevY,prevWidth,prevHeight);

    x      = prevX       = sweepArea.x;
    y      = prevY       = sweepArea.y;
    width  =  prevWidth  = sweepArea.width;
    height = prevHeight  = sweepArea.height;

    XDrawRectangle(display,fullScreen.window,fullScreen.drawgc,
	           x,y,width,height);
    {
	int cenx,ceny;
	cenx = x + width/2;
	ceny = y + height/2;
        /* draw center lines */
        XDrawLine(display,fullScreen.window,fullScreen.drawgc,
		  x-TickLength,ceny,x,ceny);
        XDrawLine(display,fullScreen.window,fullScreen.drawgc,
		  x+width,ceny, x+width+TickLength,ceny);
        XDrawLine(display,fullScreen.window,fullScreen.drawgc,
		  cenx,y+height,cenx,
		  y+height+TickLength);
    }

    if (width > 3 || height > 3) {
	switch(captureMode) {
	case CaptureArea :
	    sprintf(geometry,
	    "( %2d,%2d %4d x %4d )  Click middle mouse to end sweep",x,y,width,height);
	    break;
	case CaptureNTSC :
	    sprintf(geometry,
	    "( %2d,%2d NTSC )  Click middle mouse to end sweep",x,y);
	    break;
	case CapturePAL :
	    sprintf(geometry,
	    "( %2d,%2d PAL )  Click middle mouse to end sweep",x,y);
	    break;
	}
        XFillRectangle(display,fullScreen.window,fullScreen.drawgc,
			   x,y-23,370,19);
	XDrawString(display,fullScreen.window,fullScreen.draw2gc,
				x+2,y-9,geometry,strlen(geometry));
    }
}

#define Tolerance 5

/* depending upon the cursor's proximity with the sweep area,
   determine the tracking mode */

int
findTrackMode(x,y)
int x,y;
{
    if (captureMode == CaptureNTSC || captureMode == CapturePAL)
	return TrackCenter;

    /* see if x,y touches the current sweep area */

    /* trivial checks */
    if (x < sweepArea.x-Tolerance ||
        x > sweepArea.x+sweepArea.width+Tolerance ||
        y < sweepArea.y-Tolerance ||
        y > sweepArea.y+sweepArea.height+Tolerance) return TrackOutside;

    if (x > sweepArea.x+Tolerance && 
        x < sweepArea.x+sweepArea.width-Tolerance &&
        y > sweepArea.y+Tolerance &&
        y < sweepArea.y+sweepArea.height-Tolerance) return TrackCenter;

    if (y < sweepArea.y+Tolerance) {
	if (x < sweepArea.x+Tolerance) return TrackTopLeft;
	if (x > sweepArea.x+sweepArea.width-Tolerance) return TrackTopRight;
	return TrackTop;
    }

    if (y > sweepArea.y+sweepArea.height-Tolerance) {
	if (x < sweepArea.x+Tolerance) return TrackBottomLeft;
	if (x > sweepArea.x+sweepArea.width-Tolerance) return TrackBottomRight;
	return TrackBottom;
    }

    if (x < sweepArea.x+Tolerance) return TrackLeft;
    if (x > sweepArea.x+sweepArea.width-Tolerance) return TrackRight;

    return TrackOutside;
}
