/*----------------------------------------------------------------------------
 * 
 * file   : setting.c 
 * 
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 * 
 *---------------------------------------------------------------------------*/

#include "Xmincludes.h"
#include "globals.h"

static Arg args[40];
static int argcnt;

extern Widget buildMenu();
Widget createSettingDialog();

static int delayValues[11] = {0,1,2,3,4,5,6,7,8,9,10};

static void modeCB(),
	    delayCB();

void
popDownSetting(w,dialog)
Widget w,dialog;
{
    char *text = XmTextGetString(fileSaveTextW);
    char *c;

    if ((c = strrchr(text,'.'))) {
	/* parse the new file name */
        strcpy(fileExtension,c);
	*c = '\0';
	if (strncmp(baseFileName,text,strlen(baseFileName)))
	    strcpy(baseFileName,text);
	sprintf(fileToSave,"%s%s",baseFileName,fileExtension);
    }
    else {
	/* no extension specified */
	strcpy(fileExtension,"\0");
	if (strncmp(baseFileName,text,strlen(baseFileName)))
	    strcpy(baseFileName,text);
	sprintf(fileToSave,"%s%s",baseFileName,fileExtension);
    }

    XtFree(text);

    autoIncrement = XmToggleButtonGetState(incrementToggleW);
    XtUnmanageChild(dialog);
}

void
toggleCB(w)
Widget w;
{
    if (w == incrementToggleW) {
        autoIncrement = XmToggleButtonGetState(w);
    }
    else if (w == aspectToggleW) {
        keepAspectRatio = XmToggleButtonGetState(w);
	aspectRatio = (float)sweepArea.width/(float)sweepArea.height;
    }
    else if (w == squareToggleW) {
	squareConstraint = XmToggleButtonGetState(w);
	if (squareConstraint) {
	    if (sweepArea.width >= sweepArea.height) {  /* the smaller one wins */
		sweepArea.width = sweepArea.height;
	    }
	    else {
		sweepArea.height = sweepArea.width;
	    }
	    aspectRatio = 1.0;
	    if (sweeping)
	        drawSweep();
	}
    }
}

Widget
createSettingDialog(parent)
Widget parent;
{
    Widget dialog,workArea;
    Widget modeFrame,otherFrame;
    Widget modePB[5];
    static char *captureLabels[] = {"Capture Region",
                                    "Capture Window",
                                    "Capture FullScreen",
                                    "Capture NTSC",
                                    "Capture PAL"
		                   };
    static int cmode[] = {CaptureArea,
			  CaptureWindow,
			  CaptureFullScreen,
			  CaptureNTSC,
			  CapturePAL
			 };
    Widget modeRB,otherRC;
    Widget delayOption;
    MenuItem delaySeconds[12];
    char num[3];
    int i,n;
    Arg args[5];

    n = 0;
    XtSetArg(args[n],XmNautoUnmanage,False);n++;
    XtSetArg(args[n],XmNmarginHeight,1);n++;
    XtSetArg(args[n],XmNtitle,"ScreenShot setting");n++;
    dialog = XmCreatePromptDialog(toplevel,"selecton",args,n);
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_PROMPT_LABEL));
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_COMMAND_TEXT));
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_CANCEL_BUTTON));

    XtAddCallback(XmSelectionBoxGetChild(dialog,XmDIALOG_OK_BUTTON),
		  XmNactivateCallback,(XtPointer)popDownSetting,dialog);

    workArea = XtVaCreateManagedWidget("workArea", 
                                        xmRowColumnWidgetClass,dialog, 
					XmNmarginHeight,1,
					NULL);

    createLabel(workArea,"Capture Modes:");

    modeFrame = XtVaCreateManagedWidget("modeFrame",
                                         xmFrameWidgetClass,workArea,
					 NULL);

    modeRB    = XtVaCreateManagedWidget("modeRB",
                                         xmRowColumnWidgetClass,modeFrame,
					 XmNradioBehavior, True,
					 XmNradioAlwaysOne,      True,
					 XmNpacking,             XmPACK_TIGHT,
					 XmNmarginWidth,         0,
					 XmNmarginHeight,        0,
					 NULL);

    for (i=0;i<5;i++) {
	modePB[i] = XtVaCreateManagedWidget(captureLabels[i],
					     xmToggleButtonWidgetClass, modeRB,
					     XmNmarginHeight,        0,
					     XmNhighlightThickness,  0,
					     XmNset,                 (i==0)?True:False,
					     NULL);
	XtAddCallback(modePB[i],XmNvalueChangedCallback,
			(XtPointer)modeCB,(XtPointer)cmode[i]);
    }

    createLabel(workArea,"Other Options:");
    otherFrame = XtVaCreateManagedWidget("otherFrame",
                                          xmFrameWidgetClass,workArea,
                                          NULL);


    otherRC   = XtVaCreateManagedWidget("otherRC",
                                         xmRowColumnWidgetClass,otherFrame,
					 XmNmarginWidth,         0,
					 XmNmarginHeight,        0,
					 NULL);
    aspectToggleW  = XtVaCreateManagedWidget("Maintain Aspect Ratio",
				       xmToggleButtonWidgetClass,otherRC,
				       XmNhighlightThickness, 0,
				       XmNmarginHeight,1,
				       NULL);
    XtAddCallback(aspectToggleW,XmNvalueChangedCallback,(XtPointer)toggleCB,NULL);

    squareToggleW  = XtVaCreateManagedWidget("Maintain Square",
				       xmToggleButtonWidgetClass,otherRC,
				       XmNhighlightThickness, 0,
				       XmNmarginHeight,1,
				       NULL);
    XtAddCallback(squareToggleW,XmNvalueChangedCallback,(XtPointer)toggleCB,NULL);

    incrementToggleW   = XtVaCreateManagedWidget("Auto Increment Filename after Save",
				       xmToggleButtonWidgetClass,otherRC,
				       XmNhighlightThickness, 0,
				       XmNmarginHeight,1,
				       NULL);

    XtAddCallback(incrementToggleW,XmNvalueChangedCallback,(XtPointer)toggleCB,NULL);

    for (i=0;i<11;i++) {
        sprintf(num,"%2d",i);
	delaySeconds[i].widget      = NULL;
        delaySeconds[i].label       = strdup(num);
        delaySeconds[i].class       = &xmPushButtonGadgetClass;
        delaySeconds[i].mnemonic    = 'a';
        delaySeconds[i].accelerator = NULL;
        delaySeconds[i].accelText   = NULL;
        delaySeconds[i].callback    = delayCB;
        delaySeconds[i].callbackData= (XtPointer)delayValues[i];
        delaySeconds[i].subItems    = NULL;
    }
    delaySeconds[11].label = NULL;

    delayOption = buildMenu(otherRC,XmMENU_OPTION,"Delay seconds before snap", ' ',delaySeconds);
    XtManageChild(delayOption);

    createLabel(workArea,"Save Image File as:");

    fileSaveTextW     = XtVaCreateManagedWidget("text",
				       xmTextWidgetClass,workArea,
				       XmNmarginHeight,1,
				       XmNvalue,fileToSave,
				       NULL);
    return dialog;
}

void
delayCB(w,which)
Widget w;
int which;
{
    delaySeconds = delayValues[which];
}

void
modeCB(w,which)
Widget w;
int which;
{
    int i;
    captureMode = which;

    switch(captureMode) {
    case CaptureWindow :
    case CaptureFullScreen :
	disableSweep();
	for (i=LeftWindow;i<=TopWindow;i++)
	    XUnmapWindow(display,redline[i].window);
	break;
    case CaptureArea :
	if (sweeping) drawSweep();
	break;
    case CaptureNTSC :
	sweepArea.width  = 640;
	sweepArea.height = 486;
	sweepArea.x = (screenWidth -sweepArea.width)/2;
	sweepArea.y = (screenHeight-sweepArea.height)/2;
	if (sweeping) drawSweep();
	break;
    case CapturePAL :
	sweepArea.width  = 768;
	sweepArea.height = 576;
	sweepArea.x = (screenWidth -sweepArea.width)/2;
	sweepArea.y = (screenHeight-sweepArea.height)/2;
	if (sweeping) drawSweep();
	break;
    }
    updateMenus();
}


void
popUpSettingCB()
{
    XtManageChild(settingDialog);
}

