/*----------------------------------------------------------------------------
 * 
 * file   : ui.c 
 * 
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 * 
 *---------------------------------------------------------------------------*/

#include "Xmincludes.h"
#include "globals.h"

static Arg args[40];
static int argcnt;

extern Widget buildMenu();
static Widget createMenuBar();

extern void doCaptureCB(),
	    doSweepCB(),
	    popUpSettingCB(),
	    helpCB(),
	    previewCB();

void updateMenus(),
     doExitCB();

extern Widget createSettingDialog(),
              createHelpDialog();

void 
createUI()
{
    Widget rc;
    settingDialog = createSettingDialog(toplevel);
    helpDialog    = createHelpDialog(toplevel);
    createMenuBar(toplevel,"menub");
    createOverlayWindows();

    XtRealizeWidget(topmostlevel);
    XtRealizeWidget(toplevel);

    /* upon invokation, map the window where the cursor is */
    {
        Window ignoreWindow,ignoreChild;
        int x,y,wx,wy;
        unsigned int mask;
        XQueryPointer(display,rootWindow,&ignoreWindow,&ignoreChild,
		        &x,&y,&wx,&wy,&mask);

        XtVaSetValues(toplevel,XmNx,x,XmNy,y,NULL);
    }
    XtPopup(toplevel,XtGrabNone);

    /* update the popup menus */
    updateMenus();
    XDefineCursor(display,XtWindow(toplevel),cameraCursor);
}


static Widget
createMenuBar(parent,name)
Widget parent;
char *name;
{

    Widget menub;
    static MenuItem items[] = {
        {NULL,"Capture Region",&xmPushButtonGadgetClass,'C',"F1","F1",
             doCaptureCB,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"Sweep Region",&xmPushButtonGadgetClass,'R',"F2","F2",
             doSweepCB,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"NULL",&xmSeparatorGadgetClass,NULL,NULL,NULL,
             NULL,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"Setting...",&xmPushButtonGadgetClass,'S',NULL,NULL,
             popUpSettingCB,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"Preview",&xmPushButtonGadgetClass,'P',NULL,NULL,
             previewCB,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"Help...",&xmPushButtonGadgetClass,'H',NULL,NULL,
             helpCB,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"NULL",&xmSeparatorGadgetClass,NULL,NULL,NULL,
             NULL,(XtPointer)NULL,(MenuItem *)NULL},

        {NULL,"Exit",&xmPushButtonGadgetClass,'E',"Ctrl<Key>C","Ctrl+C",
             doExitCB,(XtPointer)NULL,(MenuItem *)NULL},

         NULL,
    };

    argcnt = 0;
    menub = XmCreateMenuBar(parent,name,args,argcnt);
    XtManageChild(menub);

    menuTitleW = buildMenu(menub,XmMENU_PULLDOWN,"ScreenShot", '@',items);

    XtVaSetValues(menuTitleW,XmNmarginHeight,0,XmNmarginWidth,0,
			     XmNshadowThickness,0,XmNlabelType,XmPIXMAP,NULL);
    XtVaSetValues(menuTitleW,XmNlabelPixmap,cameraPixmap,NULL);

    doCaptureMenuItemW = items[0].widget;
    sweepAreaMenuItemW = items[1].widget;
    previewMenuItemW   = items[4].widget;

    return(menub);
}

void
setMenuTitle(title)
char *title;
{
    XmString mstr;
    mstr  = XmStringCreateSimple(title);
    XtVaSetValues(menuTitleW,XmNbackground,greenColor.pixel,
                             XmNlabelType,XmSTRING,               
			     XmNlabelString,mstr,NULL);
    XmStringFree(mstr);
    XmUpdateDisplay(menuTitleW);
}

void
resetMenuTitle()
{

    XtVaSetValues(menuTitleW,XmNmarginHeight,0,XmNmarginWidth,0,
			     XmNshadowThickness,0,XmNlabelType,XmPIXMAP,NULL);
}

void
updateMenus()
{
    XmString mstr;
    switch(captureMode) {
    case CaptureArea :
        mstr  = XmStringCreateSimple("Capture Region");
	XtSetSensitive(sweepAreaMenuItemW,True);
	if (sweepArea.width > 1 || sweepArea.height > 1)
	    XtSetSensitive(doCaptureMenuItemW,True);
	else 
	    XtSetSensitive(doCaptureMenuItemW,False);
	break;
    case CaptureWindow :
        mstr  = XmStringCreateSimple("Capture Window");
	XtSetSensitive(doCaptureMenuItemW,True);
	XtSetSensitive(sweepAreaMenuItemW,False);
	break;
    case CaptureFullScreen :
        mstr  = XmStringCreateSimple("Capture FullScreen");
	XtSetSensitive(doCaptureMenuItemW,True);
	XtSetSensitive(sweepAreaMenuItemW,False);
	break;
    case CaptureNTSC :
        mstr  = XmStringCreateSimple("Capture NTSC");
	XtSetSensitive(sweepAreaMenuItemW,True);
	break;
    case CapturePAL :
        mstr  = XmStringCreateSimple("Capture PAL");
	XtSetSensitive(sweepAreaMenuItemW,True);
	break;
    }
    XtVaSetValues(doCaptureMenuItemW,XmNlabelString,mstr,NULL);
    XmStringFree(mstr);


    if (previewReady) {
	char str[526];
        sprintf(str,"Preview %s",fileToPreview);
        mstr  = XmStringCreateSimple(str);
        XtVaSetValues(previewMenuItemW,XmNlabelString,mstr,
				       XmNsensitive,True,
				       NULL);
	XmStringFree(mstr);
    }
    else {
        XtSetSensitive(previewMenuItemW,False);
    }
}

Widget 
createLabel(parent,text)
Widget parent;
char *text;
{
    XmString mstr;
    Widget label;

    mstr     = XmStringCreateSimple(text);
    label    = XtVaCreateManagedWidget("label",xmLabelWidgetClass,parent,
					XmNlabelString,mstr,
					XmNmarginHeight,1,
					NULL);
    XmStringFree(mstr);
    return label;
}

void
doExitCB(w)
Widget w;
{
    exit(1);
}

/* just a placeholder */
void voidFunction(){}

